﻿
Imports USBDevice1.DeviceManagement
Imports USBDevice1.FileIO
Imports USBDevice1.Hid
Imports Microsoft.Win32.SafeHandles
Imports System.Globalization
Imports System.Runtime.InteropServices
Imports System.IO
Imports Microsoft.VisualBasic.FileIO.FileSystem

Module Module1
  Public DutyCycle As Double, Frequency As Double, opzioni As UInt16, Stato As Byte
  Public TipoOnda As Byte, Ampiezza As Double
  Public do_timer As Boolean

  Public Const CommTimeout = 1000
  Public Const APP_ID = "GD's USB Function Generator"

  Public Enum USB_CMD
    CMD_NOP = 0
    CMD_GETID = 1
    CMD_GETCAPABILITIES = 2
    CMD_GETCONFIG = 3
    CMD_SETCONFIG

    CMD_GETSTATUS = 8
    CMD_SETSTATUS

    CMD_TEST = 16
    CMD_BEEP = 17

    CMD_INIT = 18
    CMD_SETFUNCTION

    CMD_WRITEEEPROM = 24
    CMD_WRITERTC = 25
    CMD_READEEPROM = 26
    CMD_READRTC = 27
    CMD_READTIMESTAMP = 28

    CMD_RESET_IF = 30
    CMD_DEBUG_ECHO = 31
    CMD_READ_IDLOCS = 63
  End Enum

  Public Enum TIPO_ONDA
    NESSUNA
    QUADRA
    TRIANGOLARE
    SAWTOOTH
    SINUSOIDALE
    IMPULSI
    RUMORE_BIANCO
    RUMORE_ROSA
  End Enum

  Private deviceNotificationHandle As IntPtr
  Private exclusiveAccess As Boolean
  Public hidHandle As SafeFileHandle
  Private hidUsage As String
  Public myUSBDeviceDetected As Boolean
  Public myDevicePathName As String
  Public readHandle As SafeFileHandle
  Public writeHandle As SafeFileHandle

  Private MyDebugging As New Debugging() ' For viewing results of API calls via Debug.Write.
  Public MyDeviceManagement As New DeviceManagement()
  Public MyHid As New Hid()

  Friend FrmMy As Form1

  ''' <summary>
  ''' Define a class of delegates that point to the Hid.ReportIn.Read function.
  ''' The delegate has the same parameters as Hid.ReportIn.Read.
  ''' Used for asynchronous reads from the device.       
  ''' </summary>

  Public Delegate Sub ReadInputReportDelegate _
      (ByVal hidHandle As SafeFileHandle, _
      ByVal readHandle As SafeFileHandle, _
      ByVal writeHandle As SafeFileHandle, _
      ByRef myDeviceDetected As Boolean, _
      ByRef readBuffer() As Byte, _
      ByRef success As Boolean)

  ' This delegate has the same parameters as AccessForm.
  ' Used in accessing the application's form from a different thread.

  Public Delegate Sub MarshalToForm _
      (ByVal action As String, _
      ByVal textToAdd As String)


  Public Function min(ByVal n1 As UInt32, ByVal n2 As UInt32) As UInt32
    Return IIf(n1 > n2, n2, n1)
  End Function
  Public Function max(ByVal n1 As UInt32, ByVal n2 As UInt32) As UInt32
    Return IIf(n1 > n2, n1, n2)
  End Function

  Public Function HIBYTE(ByVal n As UInt16) As Byte
    Return (n >> 8) And &HFF
  End Function

  Public Function LOBYTE(ByVal n As UInt16) As Byte
    Return n And &HFF
  End Function

  Public Function LOWORD(ByVal n As UInt32) As UInt16
    Return n And &HFFFF
  End Function

  Public Function HIWORD(ByVal n As UInt32) As UInt16
    Return (n >> 16) And &HFFFF
  End Function

  Public Function MAKEWORD(ByVal a As Byte, ByVal b As Byte) As UInt16
    Dim n As UInt16
    n = b
    n <<= 8
    n = n Or a
    Return n
  End Function

  Public Function MAKELONG(ByVal a As UInt16, ByVal b As UInt16) As UInt32
    Dim n As UInt32
    n = b
    n <<= 16
    n = n Or a
    Return n
  End Function

  Public Function isdigit(ByVal c As Char) As Byte
    If c >= "0" And c <= "9" Then
      Return 1
    Else
      Return 0
    End If
  End Function


  Sub OpenComm(ByVal nomeport As String)
    Dim myVid As UInteger, myPid As UInteger

    On Error GoTo comm_error

    If Mid(nomeport, 1, 3) = "USB" Then     ' bah, restrocompatibilità :)
      myVid = Val("&H" + Mid$(nomeport, 5, 4))
      myPid = Val("&H" + Mid$(nomeport, 10, 4))
    Else
    End If

    FindTheHid(myVid, mypid)
    If myUSBDeviceDetected = False Then
      GoTo comm_error
    End If

    Exit Sub

comm_error:
    Form1.ToolStripStatusLabel1.Text = "Impossibile aprire porta " + nomeport
    Beep()
    On Error GoTo 0

  End Sub

  Private Sub GetInputReportBufferSize()

    Dim numberOfInputBuffers As Int32

    Try
      ' Get the number of input buffers.

      MyHid.GetNumberOfInputBuffers _
          (hidHandle, numberOfInputBuffers)

      ' Display the result in the text box.

      'txtInputReportBufferSize.Text = CStr(numberOfInputBuffers)

    Catch ex As Exception
      DisplayException(Form1.Name, ex)
      Throw
    End Try

  End Sub

  ''' <summary>
  ''' Uses a series of API calls to locate a HID-class device
  ''' by its Vendor ID and Product ID.
  ''' </summary>
  '''         
  ''' <returns>
  '''  True if the device is detected, False if not detected.
  ''' </returns>

  Private Function FindTheHid(myVid As UInteger, myPid As UInteger)
    Dim deviceFound As Boolean
    Dim devicePathName(127) As String
    Dim hidGuid As System.Guid
    Dim memberIndex As Int32
    Dim myProductID As Int32
    Dim myVendorID As Int32
    Dim success As Boolean

    Try
      myUSBDeviceDetected = False

      ' Get the device's Vendor ID and Product ID from the form's text boxes.

      myVendorID = myVid
      myProductID = myPid         ' usb power supply

      ' ***
      ' API function: 'HidD_GetHidGuid

      ' Purpose: Retrieves the interface class GUID for the HID class.

      ' Accepts: 'A System.Guid object for storing the GUID.
      ' ***

      HidD_GetHidGuid(hidGuid)

      'Debug.WriteLine(MyDebugging.ResultOfAPICall("GetHidGuid"))
      'Debug.WriteLine("  GUID for system HIDs: " & hidGuid.ToString)

      ' Fill an array with the device path names of all attached HIDs.

      deviceFound = MyDeviceManagement.FindDeviceFromGuid(hidGuid, devicePathName)

      ' If there is at least one HID, attempt to read the Vendor ID and Product ID
      ' of each device until there is a match or all devices have been examined.

      If deviceFound Then

        memberIndex = 0

        Do
          ' ***
          ' API function:
          ' CreateFile

          ' Purpose:
          ' Retrieves a handle to a device.

          ' Accepts:
          ' A device path name returned by SetupDiGetDeviceInterfaceDetail
          ' The type of access requested (read/write).
          ' FILE_SHARE attributes to allow other processes to access the device while this handle is open.
          ' A Security structure or IntPtr.Zero. 
          ' A creation disposition value. Use OPEN_EXISTING for devices.
          ' Flags and attributes for files. Not used for devices.
          ' Handle to a template file. Not used.

          ' Returns: a handle without read or write access.
          ' This enables obtaining information about all HIDs, even system
          ' keyboards and mice. 
          ' Separate handles are used for reading and writing.
          ' ***

          hidHandle = CreateFile _
           (devicePathName(memberIndex), _
           0, FILE_SHARE_READ Or FILE_SHARE_WRITE, _
           IntPtr.Zero, OPEN_EXISTING, _
           0, 0)

          'Debug.WriteLine(MyDebugging.ResultOfAPICall("CreateFile"))
          'Debug.WriteLine("  Returned handle: " & hidHandle.ToString)

          If Not (hidHandle.IsInvalid) Then

            ' The returned handle is valid, 
            ' so find out if this is the device we're looking for.

            ' Set the Size property of DeviceAttributes to the number of bytes in the structure.

            MyHid.DeviceAttributes.Size = Marshal.SizeOf(MyHid.DeviceAttributes)

            ' ***
            ' API function:
            ' HidD_GetAttributes

            ' Purpose:
            ' Retrieves a HIDD_ATTRIBUTES structure containing the Vendor ID, 
            ' Product ID, and Product Version Number for a device.

            ' Accepts:
            ' A handle returned by CreateFile.
            ' A pointer to receive a HIDD_ATTRIBUTES structure.

            ' Returns:
            ' True on success, False on failure.
            ' ***

            success = HidD_GetAttributes(hidHandle, MyHid.DeviceAttributes)

            If success Then

              'Debug.WriteLine("  HIDD_ATTRIBUTES structure filled without error.")
              'Debug.WriteLine("  Structure size: " & MyHid.DeviceAttributes.Size)
              'Debug.WriteLine("  Vendor ID: " & Hex(MyHid.DeviceAttributes.VendorID))
              'Debug.WriteLine("  Product ID: " & Hex(MyHid.DeviceAttributes.ProductID))
              'Debug.WriteLine("  Version Number: " & Hex(MyHid.DeviceAttributes.VersionNumber))

              ' Find out if the device matches the one we're looking for.

              If (MyHid.DeviceAttributes.VendorID = myVendorID) And _
                  (MyHid.DeviceAttributes.ProductID = myProductID) Then

                '   Debug.WriteLine("  My device detected")

                ' Display the information in form's list box.

                'lstResults.Items.Add("Device detected:")
                'lstResults.Items.Add("  Vendor ID= " & Hex(MyHid.DeviceAttributes.VendorID))
                'lstResults.Items.Add("  Product ID = " & Hex(MyHid.DeviceAttributes.ProductID))

                'ScrollToBottomOfListBox()

                myUSBDeviceDetected = True

                ' Save the DevicePathName for OnDeviceChange().

                myDevicePathName = devicePathName(memberIndex)
              Else

                ' It's not a match, so close the handle.

                myUSBDeviceDetected = False

                hidHandle.Close()

              End If

            Else
              ' There was a problem in retrieving the information.

              Debug.WriteLine("  Error in filling HIDD_ATTRIBUTES structure.")
              myUSBDeviceDetected = False
              hidHandle.Close()
            End If

          End If

          ' Keep looking until we find the device or there are no devices left to examine.

          memberIndex = memberIndex + 1

        Loop Until (myUSBDeviceDetected Or (memberIndex = devicePathName.Length))

      End If

      If myUSBDeviceDetected Then

        ' The device was detected.
        ' Register to receive notifications if the device is removed or attached.

        'success = MyDeviceManagement.RegisterForDeviceNotifications _
        '    (myDevicePathName, FrmMy.Handle, hidGuid, deviceNotificationHandle)
        ' servirebbe avere il Form già aperto.. FINIRE!

        'Debug.WriteLine("RegisterForDeviceNotifications = " & success)

        ' Learn the capabilities of the device.

        MyHid.Capabilities = MyHid.GetDeviceCapabilities(hidHandle)

        If success Then

          ' Find out if the device is a system mouse or keyboard.

          hidUsage = MyHid.GetHidUsage(MyHid.Capabilities)

          ' Get the Input report buffer size.

          GetInputReportBufferSize()
          'cmdInputReportBufferSize.Enabled = True

          ' Get handles to use in requesting Input and Output reports.

          readHandle = CreateFile _
           (myDevicePathName, _
           GENERIC_READ, FILE_SHARE_READ Or FILE_SHARE_WRITE, _
           IntPtr.Zero, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, 0)

          Debug.WriteLine(MyDebugging.ResultOfAPICall("CreateFile, ReadHandle"))
          Debug.WriteLine("  Returned handle: " & readHandle.ToString)

          If readHandle.IsInvalid Then

            exclusiveAccess = True
            'lstResults.Items.Add("The device is a system " + hidUsage + ".")
            'lstResults.Items.Add("Windows 2000 and Windows XP obtain exclusive access to Input and Output reports for this devices.")
            'lstResults.Items.Add("Applications can access Feature reports only.")
            'ScrollToBottomOfListBox()

          Else
            writeHandle = CreateFile _
              (myDevicePathName, _
              GENERIC_WRITE, FILE_SHARE_READ Or FILE_SHARE_WRITE, _
              IntPtr.Zero, OPEN_EXISTING, 0, 0)

            'Debug.WriteLine(MyDebugging.ResultOfAPICall("CreateFile, WriteHandle"))
            'Debug.WriteLine("  Returned handle: " & writeHandle.ToString)

            ' Flush any waiting reports in the input buffer. (optional)

            MyHid.FlushQueue(readHandle)

          End If
        End If
      Else
        ' The device wasn't detected.

        'lstResults.Items.Add("Device not found.")
        'cmdInputReportBufferSize.Enabled = False
        'cmdOnce.Enabled = True

        'Debug.WriteLine(" Device not found.")

        'ScrollToBottomOfListBox()
      End If

      Return myUSBDeviceDetected

    Catch ex As Exception
      DisplayException(Form1.Name, ex)
      Throw
    End Try
  End Function

  Sub Sleep(ByVal n As Long)
    Dim ti As Long

    ti = Now.Millisecond + Now.Second * 1000 + n
    '        ti = ti Mod 1000
    Do

    Loop While ti > Now.Millisecond + Now.Second * 1000
  End Sub

End Module
